#ifndef _kYSEC_PPRO_H
#define _kYSEC_PPRO_H

#include <limits.h>

#define PPRO_XML_PATH	"/etc/kysec/ppro/ppro.xml"
#define XML_HASH_SIZE	64

#define KYSEC_NOTIFY_MESG_MAX 1000

#define PPRO_MONITOR_CONF "/etc/kysec/ppro/ppro.conf"

enum ppro_type {
	PPRO_SUBS = 1,	/* can't be killed */
	PPRO_KEY	/* key process, also can't be killed, */
};

struct kysec_ppro_kernel {
	char path[PATH_MAX];
	char hash[XML_HASH_SIZE + 1];
	int type;
};

#ifdef __cplusplus
extern "C" {
#endif

/**
 * 获取文件路径的hash值
 */
char *kysec_ppro_get_path_hash(const char *filepath);

/**
 * 添加一个程序信息节点到xml文件
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 * 返回值: 0:成功，1:包含该@path的节点已经存在，-1:出错
 */
extern int kysec_ppro_add(char *path, int type);

/* 从xml文件中删除一个节点
 * @path: 程序路径
 * 返回值: 0:成功，-1:出错
 */
extern int kysec_ppro_remove(char *path);

/* 更新xml文件的某一个节点
 * @path: 程序路径
 * @type: 保护类型，enum ppro_type
 *
 * 返回值: 0:成功，1:xml文件中不存在路径为@path的节点，-1:失败
 */
extern int kysec_ppro_update(char *path, int type);

/**
 * 获取指定路径path在xml文件中的节点数据
 */
extern struct kysec_ppro_kernel *kysec_ppro_read_node_with_path(char *filepath);

/* 获取xml文件中所有节点数据
 * 返回值: kysec_ppro_kernel指针数组，大小为@count
 */
extern struct kysec_ppro_kernel *kysec_ppro_read_all_data(int *count);

/*
 * 获取 ppro.conf 中 action 的值，调用后需释放 action
 */
extern int kysec_ppro_get_action(char **action);

#ifdef __cplusplus
}
#endif

#endif // _kYSEC_PPRO_H
